//<c>   Copyright 1986-2000 by Gerry J. Danen; all rights reserved.

//<t>   File Check

/*      Modification History
        --------------------
        GJD V1.00   04-Oct-1986 Written
        GJD V1.10   12-Apr-1988 Consider <FF> to be an ASCII char
        GJD V1.11   15-Dec-1988 Increased read-buffer size
*/

/*      This program analyzes a file.  The output includes a count for each
        character that appears in the file.  Output may be redirected to a
        file or to the printer.  FCHECK will read hidden and system files.

            Usage:  FCHECK filename [>outfile]

        Full pathnames are supported for the filename.


        Compiler:   Microsoft C
        Switches:   none
*/
#include"gd_tools.h"


#define pgVERS_ "V2.20"
#define pgTITL_ "File Analysis"
#define pgCOPR_ "1986"


//      Global variables

#define BLOCKSZ (BUFSIZ*16)

static  UCHAR   pathname[64];
static  INT16   ascii_file ;
static  UINT32  ascii, non_ascii, special, bytecount, no_of_lines ;
static  UINT32  chk[256] ;              // 256 values: 0-255
static  UCHAR   work[BLOCKSZ+1];
static  size_t  longline = 0 ;          // size of longest line


//      Function definitions

void    FCHECK(int ,unsigned char *,unsigned long);
void    DoLines(void);
void    Report(void);
INT16   ReadBlock(FILE *,unsigned char *,int ,int *);
void    Dashes(void);
void    help(void);
void    NoSuchFile(char *);


//      Collect info from input buffer

void    FCHECK( INT16 l, UCHAR *inbuffer, UINT32 bytecount )
{
    UCHAR   ch;
    UINT16  cnt;

    if ( l != 0 )                       // skip if buffer empty
    {
        for ( cnt = 0 ; cnt < (UINT16) l ; cnt++ )
        {
            ch = *inbuffer++;           // extract char, advance pointer
            if ( (ch < 32) || (ch > 126) )
            {
                if ( (ch == '\t') || (ch == '\r') || (ch == '\n') || (ch == '\f') )
                    special += 1L ;
                else
                    non_ascii += 1L ;
            }
            else
                ascii += 1L ;
            chk[ch] += 1L ;             // update array
        }
    }
}


//      Count lines in ascii file

void    DoLines(void)
{
    FILE    *fdin;
    UCHAR   *ptr;

    if ((fdin = fopen(pathname,"r")) == NULL)
    {
        fprintf( stderr, "Could not reopen file %s\007\n", pathname );
        exit(1);
    }
    do  {
        ptr = fgets( work, BLOCKSZ, fdin );
        if ( ptr != NULL )
        {
            no_of_lines++ ;
            if ( work[strlen(work)-1] == '\n' )
                work[strlen(work)-1] = EOS ;
            if ( strlen(work) > longline )
                longline = strlen(work);
        }
    }   while ( ptr != NULL );
}


//      Report statistics

void    Report(void)
{
    INT16   i;
    UCHAR   ch;

    for ( i=0; i < 256; i++ )
    {
        ch = (char) i;
        if ( chk[i] != 0L )
        {
            fprintf( stdout, "\nASCII code %03d:%9lu time%c", i, chk[i], (chk[i]==1L ? ' ' : 's') );
            if ( isprint(ch) )
                fprintf( stdout, " - %c", ch );
            if ( ch == ' ' )
                fprintf( stdout, "\b<space>" );
            if ( iscntrl(ch) )
            {
                if ( ch > 26 )
                    fprintf( stdout, " - CTRL" );
                else
                    fprintf( stdout, " - CTRL-%c", ch-1+'A' );
                switch ( ch )
                {
                    case '\t' :
                        fprintf( stdout, " <TAB>" );
                        break ;
                    case '\r' :
                        fprintf( stdout, " <CR>" );
                        break ;
                    case '\n' :
                        fprintf( stdout, " <LF>" );
                        break ;
                    case '\f' :
                        fprintf( stdout, " <FF>" );
                        break ;
                    case 26 :
                        fprintf( stdout, " <end-of-file>" );
                        break ;
                }
            }
            if ( i > 127 )
                fprintf( stdout, " - Extended ASCII" );
        }
    }
    if ( (non_ascii > 1) || (chk[26] > 1L) || ((non_ascii == 1) && (chk[26] != 1L)) )
        ascii_file = FALSE ;
    else
        ascii_file = TRUE ;
    Dashes();
    fprintf( stdout, "\n%lu Bytes in file %s", bytecount, pathname );
    if ( ! ascii_file )
        fprintf( stdout, " (probably not an ASCII file)\007" );
    Dashes();
    fprintf( stdout, "\n          Ascii : %lu", ascii );
    fprintf( stdout, "\nTab, cr, lf, ff : %lu", special );
    fprintf( stdout, "\n      Non-Ascii : %lu", non_ascii );
    if ( non_ascii && chk[26] )
        fprintf( stdout, ", of which %lu %s ^Z", chk[26], (chk[26]==1L ? "is" : "are") );
    if ( ascii_file )
    {
        fprintf( stdout, "\n          Lines : %lu", no_of_lines );
        fprintf( stdout, "\n   Longest line : %u", longline );
    }
    Dashes();
    fprintf( stdout, "\n" );            // final cr/lf
}


//      Read block from input file - standard routine

INT16   ReadBlock( FILE *fdin, UCHAR *str, INT16 maxchar, INT16 *endfile )
/*
    ReadBlock:  Read (maxchar-1) bytes from "fdin".

    -- Get string str from fdin, maximum (maxchar-1) characters.

    -- When EOF is reached, endfile becomes TRUE.

    -- Does NOT flush characters if maxchar is reached.

    -- Append \0 at end of buffer.
*/
{
    INT16   len;
    UCHAR   ch;

    maxchar--;                          // leave room for the \0 terminator
    for ( len=0 ; (len < maxchar) && (!feof(fdin)) ; len++ )
    {
        ch = (UCHAR) getc(fdin);        // get character from file "fdin"
        *str++ = ch;                    // store in string
    }
    *str = '\0';                        // string terminator
    if ( feof( fdin ) )                 // check for EOF
    {
        len--;
        *endfile=TRUE;
    }
    else
        *endfile=FALSE;
    return( len );
}


//      MISCELLANEOUS FUNCTIONS

void    Dashes(void)
{
    fprintf( stdout, "\n-------------------------------------------------------------------------------\r" );
}


void    help(void)
{
    fprintf( stderr, "\n\nUsage:\t  %s filename [>outfile]\n\n"
                     "\t\t filename - full pathname supported\n"
                     "\t\t >outfile - redirect to file or printer\n\n"
                     "\t  %s will read hidden and system files also!\n\n",
                     get_global_program_name(), get_global_program_name() );
    exit( 1 );
}


void    NoSuchFile( char *filename )
{
    fprintf( stderr, "\n\nFile %s not found\007\n", filename );
}


void    main( int argc, char *argv[] )
{
    FILE    *fdin;
    INT16   ch_ctr, endfile;

    tt_init( SHAREWARE_VERSION, 0, pgVERS_, pgTITL_, pgCOPR_ );
    logon(0);

    if ( argc != 2 )                            // do we have file name?
        help();                                 // nope!

    strcpy( pathname, argv[1] );
    _strupr( pathname );
    if ( (fdin = fopen(pathname,"rb")) == NULL )
    {
        NoSuchFile(pathname);
        help();
    }

    fprintf( stdout, "\nFile: %s", pathname );
    Dashes();
    for (;;)
    {
        ch_ctr = ReadBlock( fdin, work, BLOCKSZ+1, &endfile );
        FCHECK( ch_ctr, work, bytecount );
        bytecount = bytecount + ch_ctr;
        if ( endfile )                          // EOF
        {
            _fcloseall();                       // close any open files
            if ( non_ascii < 2 )                // is ascii file for sure
                DoLines();                      // count lines
            Report();                           // stats
            exit( 0 );                          // done
        }
    }
}
